using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace SpiDotNet
{
    class NativeMethods
    {
        [DllImport("coredll.dll", EntryPoint = "CreateFile", SetLastError = true)]
        public static extern IntPtr CreateFile
        (
            String lpFileName,
            UInt32 dwDesiredAccess,
            UInt32 dwShareMode,
            IntPtr lpSecurityAttributes,
            UInt32 dwCreationDisposition,
            UInt32 dwFlagsAndAttributes,
            IntPtr hTemplateFile
        );

        [DllImport("Coredll.dll")]
        public static extern int CloseHandle(IntPtr handle);

        [DllImport("coredll.dll", SetLastError = true)]
        private static extern bool DeviceIoControl
        (
            IntPtr hDevice,
            uint dwIoControlCode,
            IntPtr lpInBuffer,
            uint nInBufferSize,
            IntPtr lpOutBuffer,
            uint nOutBufferSize,
            ref uint lpBytesReturned,
            IntPtr lpOverlapped
        );
     
        // IOCTL creation specific constants
        internal const uint METHOD_BUFFERED = 0;
        internal const uint FILE_WRITE_ACCESS = 0x0002;
        internal const uint FILE_DEVICE_UNKNOWN = 0x00000022;
        internal const uint FILE_ANY_ACCESS = 0;
        
        public static uint CTL_CODE_DEVICE_UNKNOWN(uint uiFunction)
        {
            return ((FILE_DEVICE_UNKNOWN << 16) | (FILE_ANY_ACCESS << 14) | ((uiFunction) << 2) | METHOD_BUFFERED);
        }

        public static bool SendIOCTL(IntPtr hDrv, uint ioctl, Object inputData, uint inputParametersSize)
        {
            UInt32 stam=0;
            return SendIOCTL(hDrv, ioctl, inputData, inputParametersSize, out stam, 0);
        }
        
        public static bool SendIOCTL(IntPtr hDrv, uint ioctl, Object inputData, uint inputParametersSize, out UInt32 outputData, uint outputParametersSize)
        {
            uint bytesReturned = 0;
            // Initialize unmanged memory to hold the struct.
            IntPtr inputParametersPtr = IntPtr.Zero;
            IntPtr outputParametersPtr = IntPtr.Zero;
            try
            {
                if (inputParametersSize > 0)
                {
                    inputParametersPtr = Marshal.AllocHGlobal((int)inputParametersSize);
                    if (inputParametersSize > 4)
                        Marshal.StructureToPtr(inputData, inputParametersPtr, false);
                    else
                        Marshal.WriteInt32(inputParametersPtr, Convert.ToInt32(inputData));

                }
                if (outputParametersSize > 0)
                {
                    outputParametersPtr = Marshal.AllocHGlobal((int)outputParametersSize);
                }
                bool res = NativeMethods.DeviceIoControl(hDrv, ioctl,
                                           inputParametersPtr, inputParametersSize,
                                           outputParametersPtr, outputParametersSize,
                                           ref bytesReturned, IntPtr.Zero);
                if (!res)
                {
                    // Throw Win32 exception with error code
                    int err = Marshal.GetLastWin32Error();
                    throw new System.ComponentModel.Win32Exception(err);
                }
                if (outputParametersSize > 0)
                {
                    outputData = (UInt32)Marshal.ReadInt32(outputParametersPtr);
                }
                else
                    outputData = 0;
            }
            catch (Exception ex)
            {
                // throw all other exceptions to upper levels
                throw ex;
            }
            finally
            {
                if (inputParametersSize > 0)
                    Marshal.FreeHGlobal(inputParametersPtr);
                if (outputParametersSize > 0)
                    Marshal.FreeHGlobal(outputParametersPtr);
            }
            return true;
        }

        public static bool SendSpiIOCTL(IntPtr hDrv, uint ioctl, UInt32 inputData, uint inputParametersSize, out UInt32 outputData, uint outputParametersSize)
        {
            uint bytesReturned = 0;
            // Initialize unmanged memory to hold the struct.
            IntPtr inputParametersPtr = IntPtr.Zero;
            IntPtr outputParametersPtr = IntPtr.Zero;
            try
            {
                if (inputParametersSize > 0)
                {
                    inputParametersPtr = Marshal.AllocHGlobal((int)inputParametersSize);
                    Marshal.WriteInt32(inputParametersPtr, Convert.ToInt32(inputData));
                }
                if (outputParametersSize > 0)
                {
                    outputParametersPtr = Marshal.AllocHGlobal((int)outputParametersSize);
                }
                bool res = NativeMethods.DeviceIoControl(hDrv, ioctl,
                                           outputParametersPtr, outputParametersSize,
                                           inputParametersPtr, inputParametersSize,
                                           ref bytesReturned, IntPtr.Zero);
                if (!res)
                {
                    // Throw Win32 exception with error code
                    int err = Marshal.GetLastWin32Error();
                    throw new System.ComponentModel.Win32Exception(err);
                }
                if (outputParametersSize > 0)
                {
                    outputData = (UInt32)Marshal.ReadInt32(outputParametersPtr);
                }
                else
                    outputData = 0;
            }
            catch (Exception ex)
            {
                // throw all other exceptions to upper levels
                throw ex;
            }
            finally
            {
                if (inputParametersSize > 0)
                    Marshal.FreeHGlobal(inputParametersPtr);
                if (outputParametersSize > 0)
                    Marshal.FreeHGlobal(outputParametersPtr);
            }
            return true;
        }

        public static UInt32 GetIOCTL(IntPtr hDrv, uint ioctl, uint value)
        {
            UInt32 inVal = value;
            UInt32 retVal = 0;

            SendIOCTL(hDrv, ioctl, inVal, (uint)Marshal.SizeOf(inVal), out retVal, (uint)Marshal.SizeOf(retVal));
            return retVal;
        }
    }
}
