using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace LCDAdapt
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new NavigateForm());
        }
    }

    public enum Platforms { CM_X300 , CM_T35};
    #region private struct declaration
    public struct BaseSettings
    {
        /// <summary>
        /// VSW(vertical)/HSW
        /// </summary>
        public uint SyncPulseWidth;
        public uint SyncPolarity;
        /// <summary>
        /// EFW(vertical)/ELW
        /// </summary>
        public uint BackPorch;
        /// <summary>
        /// BFW(vertical)/BLW
        /// </summary>
        public uint FrontPorch;
    };
    #region cm-t35
    public struct CM_T35
    {
        public uint Height;
        public uint Width;
        public BaseSettings Horizontal;
        public BaseSettings Vertical;
        public uint PixelClockFrequency;
        public uint PixelClockPolarity;
        public uint IsDVIEnabled;
        public uint IsLVDSEnabled;
    }
    #endregion
    #region cm-x300
    public struct PCD
    {
        public int d0cs;
        public int _39m;
        public int _104m;
        public int _156m;
        public int _208m;
    }
    public struct CM_X300
    {
        public PCD pcd;
        public uint height;
        public uint width;
        public uint bpp;
        public BaseSettings horizontal;
        public BaseSettings vertical;
        public int pixelclockpolarity;
        public int dataenablepolarity;
    }
    #endregion
    #endregion

    [StructLayout(LayoutKind.Explicit)]
    public struct PLATFORM_UNION
    {
        [FieldOffset(0)]
        public CM_X300 cm_x300;
        [FieldOffset(0)]
        public CM_T35 cm_t35;
    }
    /// <summary>
    /// This struct holds the data to pass between forms
    /// </summary>   
    [StructLayout(LayoutKind.Sequential)]
    public struct LcdSettings
    {
        public Platforms platform;
        public bool IsValid;
        public PLATFORM_UNION plat;
    }

    public class BaseForm : Form
    {
        #region Form information interchange
        public delegate bool SetLcdSettings(LcdSettings settings);
        public delegate bool GetLcdSettings(out LcdSettings settings);
        #endregion
    }
}